//
//  io_helper.hpp
//  Computational Geometry - 2018
//
//  Requires C++11 or later.
//
//  Created by Maximilian Katzmann on 09.10.18.
//

#ifndef io_helper_hpp
#define io_helper_hpp

#include <fstream>
#include <iostream>
#include <limits>
#include <math.h>
#include <sstream>
#include <string>
#include <vector>

struct Point {

  double x = 0.0;
  double y = 0.0;

  Point(double x, double y) : x(x), y(y) {}

  /**
   * Make it easier to print a point to the console.
   */
  friend std::ostream &operator<<(std::ostream &os, const Point &p) {
    return os << p.to_string();
  }

  std::string to_string() const {
    return std::string("(") + std::to_string(this->x) + ", " +
           std::to_string(this->y) + ")";
  }
};

class IOHelper {

public:
  /**
   * Given a file name, tries to read each line in the file as a
   * two-dimensional point and adds it to the points vector.
   */
  static void read_points_from_file(const std::string &file_name,
                                    std::vector<Point> &points) {

    /**
     * Open the file.
     */
    std::ifstream file(file_name.c_str());

    /**
     * The variable that will hold each line.
     */
    std::string line;

    /**
     * Keep track of the line number so we can show it when an error
     * occurred.
     */
    int current_line_number = 1;

    /**
     * Iterate through the lines.
     */
    while (std::getline(file, line)) {

      std::vector<double> numbers_in_line;
      double number;

      std::stringstream stream(line);

      while (stream >> number) {
        numbers_in_line.push_back(number);
      }

      /**
       * A point consists of exactly two numbers.
       */
      if (numbers_in_line.size() == 2) {

        /**
         * Add the point.
         */
        points.push_back(Point(numbers_in_line[0], numbers_in_line[1]));

      } else {
        std::cerr << "Invalid number of numbers in line: "
                  << current_line_number << ". Skipping this line."
                  << std::endl;
      }

      /**
       * Increase the line number.
       */
      ++current_line_number;
    }
  }

  /**
   * Determines the svg representation of a polygon and stores it
   * in the passed parameter.
   */
  static void svg_representation_of_polygon(const std::vector<Point> &polygon,
                                            const std::string &color,
                                            const double stroke_width,
                                            const bool is_closed,
                                            std::string &representation) {
    /**
     * The beginning of the path.
     */
    representation = "<path d =\"";

    if (polygon.size() > 0) {

      /**
       * We use height - ... in the y-coordinate, as SVG has the
       * origin at the top.
       */
      representation += "M " + std::to_string(polygon[0].x) + "," +
                        std::to_string(polygon[0].y) + " ";

      for (int index = 1; index < (int)polygon.size(); ++index) {
        representation += "L " + std::to_string(polygon[index].x) + "," +
                          std::to_string(polygon[index].y) + " ";
      }

      /**
       * Close the path.
       */
      if (is_closed) {
        representation += "Z";
      }
    }

    representation += "\" stroke = \"" + color + "\" stroke-width = \"" +
                      std::to_string(stroke_width) + "\" fill=\"none\"/>\n";
  }

  /**
   * Takes a set of points and polygons and generates the SVG code
   * representing the passed objects.  (Polygons are always drawn
   * closed.)
   *
   * We scale everything such that the content in the SVG is not too
   * small.
   *
   */
  static void
  svg_representation(const std::vector<Point> &points,
                     const std::vector<std::vector<Point>> &polygons,
                     std::string &representation,
                     const double scale = 50.0) {

    /**
     * Determine the maximum/minimum coordinates for both axis in
     * order to calculate the width and height of the canvas.
     */
    double x_max = std::numeric_limits<double>::min();
    double x_min = std::numeric_limits<double>::max();
    double y_max = std::numeric_limits<double>::min();
    double y_min = std::numeric_limits<double>::max();

    /**
     * Find min/max in points.
     */
    for (const Point &point : points) {
      if (point.x > x_max) {
        x_max = point.x;
      }

      if (point.x < x_min) {
        x_min = point.x;
      }

      if (point.y > y_max) {
        y_max = point.y;
      }

      if (point.y < y_min) {
        y_min = point.y;
      }
    }

    /**
     * Find min/max in polygons.
     */
    for (const std::vector<Point> &polygon : polygons) {
      for (const Point &point : polygon) {
        if (point.x > x_max) {
          x_max = point.x;
        }

        if (point.x < x_min) {
          x_min = point.x;
        }

        if (point.y > y_max) {
          y_max = point.y;
        }

        if (point.y < y_min) {
          y_min = point.y;
        }
      }
    }

    /**
     * We add a margin on the sides of the SVG.
     */
    double margin = 0.1;

    x_min = floor(x_min) - 1;
    x_max = ceil(x_max) + 1;
    y_min = floor(y_min) - 1;
    y_max = ceil(y_max) + 1;

    /**
     * Width and height of the canvas.
     */
    double width = scale * (x_max - x_min + (2.0 * margin));
    double height = scale * (y_max - y_min + (2.0 * margin));

    /**
     * All points will be shifted to accommodate for the margin.  And
     * we need to shift everything, such that there are no negative
     * coordinates.
     */
    double x_offset = margin - x_min;
    double y_offset = margin - y_min;

    /**
     * SVG Header
     */
    representation =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE svg PUBLIC "
        "\"-//W3C//DTD SVG 1.1//EN\" "
        "\"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n\n<svg "
        "xmlns=\"http://www.w3.org/2000/svg\"\nxmlns:xlink=\"http://"
        "www.w3.org/1999/xlink\" "
        "xmlns:ev=\"http://www.w3.org/2001/xml-events\"\nversion=\"1.1\" "
        "baseProfile=\"full\"\nwidth=\"" +
        std::to_string(width) + "\" height=\"" + std::to_string(height) +
        "\">\n\n";

    /**
     * Drawing the grid.
     */
    const std::string grid_color = "blue";
    const double grid_stroke_width = 0.005;
    const double grid_stroke_width_main = 0.02;

    /**
     * Grid lines in x-direction
     */
    for (int y_index = y_min; y_index <= y_max; ++y_index) {
      std::string axis_string;
      double converted_y = height - scale * (y_index + y_offset);
      double stroke_width = scale * (y_index == 0 ? grid_stroke_width_main : grid_stroke_width);
      const std::vector<Point> axis_points = {Point(0.0, converted_y),
                                              Point(width, converted_y)};
      IOHelper::svg_representation_of_polygon(axis_points, grid_color,
                                              stroke_width, false, axis_string);

      representation += axis_string;
    }

    /**
     * Grid lines in y-direction.
     */
    for (int x_index = x_min; x_index <= x_max; ++x_index) {
      std::string axis_string;
      double converted_x = scale * (x_index + x_offset);
      double stroke_width = scale * (x_index == 0 ? grid_stroke_width_main : grid_stroke_width);
      const std::vector<Point> axis_points = {Point(converted_x, 0.0),
                                              Point(converted_x, height)};
      IOHelper::svg_representation_of_polygon(axis_points, grid_color,
                                              stroke_width, false, axis_string);

      representation += axis_string;
    }

    /**
     * Drawing the polygons.
     */
    const std::string polygon_color = "black";
    const double stroke_width = 0.05;

    for (const std::vector<Point> &polygon : polygons) {

      /**
       * Convert the points of the polygon to points on the canvas.
       */
      std::vector<Point> converted_polygon;
      for (const Point &point : polygon) {
        Point canvas_point(scale * (point.x + x_offset),
                           height - scale * (point.y + y_offset));
        converted_polygon.push_back(canvas_point);
      }

      /**
       * Get the svg representation of the polygon.
       */
      std::string polygon_string;
      IOHelper::svg_representation_of_polygon(converted_polygon, polygon_color,
                                              scale * stroke_width, true,
                                              polygon_string);

      representation += polygon_string;
    }

    /**
     * Drawing the points.
     */
    const double point_radius = 0.05;
    const std::string point_color = "red";

    for (const Point &point : points) {

      /**
       * We use height - ... in the y-coordinate, as SVG has the
       * origin at the top.
       */
      std::string point_string =
          "<circle cx=\"" + std::to_string(scale * (point.x + x_offset)) +
          "\" cy=\"" + std::to_string(height - scale * (point.y + y_offset)) +
          "\" r=\"" + std::to_string(scale * point_radius) + "\" fill=\"" +
          point_color + "\" stroke=\"" + point_color + "\" stroke-width=\"" +
          std::to_string(scale * stroke_width) + "\"/>\n";
      representation += point_string;
    }

    /**
     * SVG Footer
     */
    representation += "</svg>\n";
  }
}
;

#endif /* io_helper_hpp */
