#
#  io_helper.py
#  Computational Geometry - 2018
#
#  Created by Maximilian Katzmann on 10.10.18.
#

import math

# Class that represents a point
class Point(object):
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __str__(self):
        return "(x = " + str(self.x) + ", y = " + str(self.y) + ")"

class IOHelper(object):

    # Returns a list of points representing the coordinates in the file with
    # the passed file_name.
    def read_points_from_file(file_name):

        # The list of points that will be returned.
        points = []

        # Open the file and read all lines.
        with open(file_name, "rb") as file:
            lines = file.readlines()

            # Iterate the lines.
            line_number = 1
            for line in lines:
                line = line.strip()

                # Skip empty lines.
                if not line:
                    continue

                line_components = line.split()

                # A line needs to consist of exactly 2 components.
                if len(line_components) == 2:
                    point = Point(float(line_components[0]), float(line_components[1]))
                    points.append(point)
                else:
                    print("Invalid number of numbers in line: " + str(line_number) + ". Skipping this line.")

                line_number = line_number + 1

        return points

    # Returns a string that represents the passed polygon as an SVG path.
    def svg_representation_of_polygon(polygon, color, stroke_width, is_closed):

        path_string = "<path d =\""

        if len(polygon) > 0:
            path_string += "M " + str(polygon[0].x) + "," + str(polygon[0].y) + " "

            for index in range(1, len(polygon)):
                path_string += "L " + str(polygon[index].x) + "," + str(polygon[index].y) + " "

        if is_closed:
            path_string += "Z"

        path_string += "\" stroke = \"" + color + "\" stroke-width = \"" + str(stroke_width) + "\" fill=\"none\"/>"
        return path_string

    def svg_representation(points, polygons, scale = 50.0):

        # Determine the minimum/maximum coordinates for both axis in order to
        # calculate the width and height of the canvas.
        x_max = -float("inf")
        x_min = float("inf")
        y_max = -float("inf")
        y_min = float("inf")

        # Find min/max in points.
        for point in points:
            if point.x > x_max:
                x_max = point.x
            if point.x < x_min:
                x_min = point.x
            if point.y > y_max:
                y_max = point.y
            if point.y < y_min:
                y_min = point.y

        # Find min/max in polygons.
        for polygon in polygons:
            for point in polygon:
                if point.x > x_max:
                    x_max = point.x
                if point.x < x_min:
                    x_min = point.x
                if point.y > y_max:
                    y_max = point.y
                if point.y < y_min:
                    y_min = point.y

        x_min = math.floor(x_min) - 1
        x_max = math.ceil(x_max) + 1
        y_min = math.floor(y_min) - 1
        y_max = math.ceil(y_max) + 1

        # We add a margin on the sides of the SVG.
        margin = 0.1

        # Width and height of the canvas.
        width = scale * (x_max - x_min + (2.0 * margin));
        height = scale * (y_max - y_min + (2.0 * margin));

        # All points will be shifted to accommodate for the margin.  And
        # we need to shift everything, such that there are no negative
        # coordinates.
        x_offset = margin - x_min
        y_offset = margin - y_min

        # SVG Header
        representation = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE svg PUBLIC " +\
              "\"-//W3C//DTD SVG 1.1//EN\" " +\
              "\"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n\n<svg " +\
              "xmlns=\"http://www.w3.org/2000/svg\"\nxmlns:xlink=\"http://" +\
              "www.w3.org/1999/xlink\" " +\
              "xmlns:ev=\"http://www.w3.org/2001/xml-events\"\nversion=\"1.1\" " +\
              "baseProfile=\"full\"\nwidth=\"" + str(width) + "\" height=\"" + str(height) + "\">\n"

        # Drawing the grid.
        grid_color = "blue";
        grid_stroke_width = 0.005;
        grid_stroke_width_main = 0.02;

        # Grid lines in x-direction
        for y_index in range(y_min, y_max + 1, 1):
            converted_y = height - scale * (y_index + y_offset)
            stroke_width = scale * (grid_stroke_width_main if y_index == 0 else grid_stroke_width)
            axis_points = [Point(0.0, converted_y), Point(width, converted_y)]
            axis_string = IOHelper.svg_representation_of_polygon(axis_points, grid_color, stroke_width, False)
            representation += axis_string

        # Grid lines in y-direction
        for x_index in range(x_min, x_max + 1, 1):
            converted_x = scale * (x_index + x_offset)
            stroke_width = scale * (grid_stroke_width_main if x_index == 0 else grid_stroke_width)
            axis_points = [Point(converted_x, 0.0), Point(converted_x, height)]
            axis_string = IOHelper.svg_representation_of_polygon(axis_points, grid_color, stroke_width, False)
            representation += axis_string

        # Drawing the polygons.
        polygon_color = "black"
        stroke_width = 0.05

        for polygon in polygons:
            converted_polygon = []

            for point in polygon:
                canvas_point = Point(scale * (point.x + x_offset), height - scale * (point.y + y_offset))
                converted_polygon.append(canvas_point)

            # Get the svg representation of the polygon.
            polygon_string = IOHelper.svg_representation_of_polygon(converted_polygon, polygon_color, scale * stroke_width, True)
            representation += polygon_string

        # Drawing the points.
        point_radius = 0.05
        point_color = "red"

        for point in points:
            representation += "<circle cx=\"" + str(scale * (point.x + x_offset)) + "\" cy=\"" + str(height - scale * (point.y + y_offset)) + "\" r=\"" + str(scale * point_radius) + "\" fill=\"" + str(point_color) + "\" stroke=\"" + str(point_color) + "\" stroke-width=\"" + str(scale * stroke_width) + "\"/>"

        # SVG Footer
        representation += "\n</svg>\n"

        return representation
