#include <algorithm>
#include <cmath>
#include <fstream>
#include <functional>
#include <iostream>
#include <vector>
#include <numeric>

struct Point {
  int x;
  int y;
};

struct Rectangle {
  int x_min;
  int x_max;
  int y_min;
  int y_max;
};

// implicit representation of a full binary tree with n leaves; each
// node is given by its layer and its position (from left to right) in
// this layer
class TreeNode {
 public:
  TreeNode(int layer, int pos) : layer_(layer), pos_(pos) {}

  int layer() { return layer_; }
  int pos() { return pos_; }

  TreeNode left_child() const { return TreeNode(layer_ + 1, 2 * pos_); }
  TreeNode right_child() const { return TreeNode(layer_ + 1, 2 * pos_ + 1); }
  TreeNode parent() const { return TreeNode(layer_ - 1, pos_ / 2); }

  bool is_left_child() const { return pos_ % 2 == 0; }
  bool is_right_child() const { return pos_ % 2 == 1; }

  TreeNode leftmost_leaf(int n) const;
  TreeNode rightmost_leaf(int n) const;

  bool node_exists(int n) const { return leftmost_leaf(n).pos() < n; }

 private:
  int layer_;
  int pos_;
};

class RangeTree {
 public:
  RangeTree(std::vector<Point> points);
  std::vector<int> range_query(const Rectangle& R) const;

  static int max_layer(int n) { return std::ceil(std::log2(n)); }

  void print() const;

 private:
  int n() const { return points_.size(); }
  int max_layer() const { return RangeTree::max_layer(n()); }

  // finds the leftmost leaf whose point has x-coordinate at least
  // x_coord if left_most is true; rightmost leaf whose x-coordinate
  // is at most x_coord, otherwise
  TreeNode x_query(int x_coord, bool leftmost) const;

  // query the rectangle in the y-array stored for the given tree node
  // and add the resulting points to the result
  void y_query(const Rectangle& R, TreeNode node,
               std::vector<int>& result) const;

  std::vector<Point> points_;

  // the points (identified by their position in points_) sorted by x
  // coordinate
  std::vector<int> point_ids_x_sorted_;

  // for each layer and position in this layer (aka for each
  // TreeNode), one array sorted by y-coordinates
  std::vector<std::vector<std::vector<int>>> point_ids_y_sorted_;
};

//////////////////////////////////////////////////////////////////////
// implementation

TreeNode TreeNode::leftmost_leaf(int n) const {
  int steps = RangeTree::max_layer(n) - layer_;
  return TreeNode(RangeTree::max_layer(n), pos_ * std::pow(2, steps));
}

TreeNode TreeNode::rightmost_leaf(int n) const {
  int steps = RangeTree::max_layer(n) - layer_;
  int number_of_leaves = std::pow(2, steps);
  int leaf_pos = leftmost_leaf(n).pos() + number_of_leaves - 1;
  return TreeNode(RangeTree::max_layer(n), std::min(leaf_pos, n - 1));
}

RangeTree::RangeTree(std::vector<Point> points) : points_(points) {
  // sort the points by x (leaves of the tree)
  point_ids_x_sorted_.resize(n());
  std::iota(point_ids_x_sorted_.begin(), point_ids_x_sorted_.end(), 0);
  std::sort(point_ids_x_sorted_.begin(), point_ids_x_sorted_.end(),
            [&](int id1, int id2) { return points_[id1].x < points_[id2].x; });

  // create the y-arrays for each tree node (we don't compute the tree
  // explicitly, it is implicitly given by the number of points n)
  point_ids_y_sorted_.resize(max_layer() + 1);
  int layer_size = 1;
  for (int layer = 0; layer <= max_layer(); ++layer) {
    point_ids_y_sorted_[layer].resize(layer_size);
    for (int pos = 0; pos < layer_size; ++pos) {
      // find leftmost and rightmost leaf of current node
      TreeNode node(layer, pos);
      if (!node.node_exists(n())) {
        break;
      }
      auto l = point_ids_x_sorted_.begin() + node.leftmost_leaf(n()).pos();
      auto r = point_ids_x_sorted_.begin() + node.rightmost_leaf(n()).pos() + 1;
      // copy elements in this range into y-array and sort by y
      point_ids_y_sorted_[layer][pos] = std::vector<int>(l, r);
      std::sort(point_ids_y_sorted_[layer][pos].begin(),
                point_ids_y_sorted_[layer][pos].end(), [&](int id1, int id2) {
                  return points_[id1].y < points_[id2].y;
                });
    }
    layer_size *= 2;
  }
}

std::vector<int> RangeTree::range_query(const Rectangle& R) const {
  std::vector<int> result;

  TreeNode x_min_node = x_query(R.x_min, true);
  TreeNode x_max_node = x_query(R.x_max, false);

  // handle some special cases of (almost) empty ranges
  if (x_min_node.pos() > x_max_node.pos()) {
    return result;
  }
  if (x_min_node.pos() == x_max_node.pos()) {
    Point p = points_[point_ids_x_sorted_[x_min_node.pos()]];
    if (R.x_min <= p.x && p.x <= R.x_max && R.y_min <= p.y && p.y <= R.y_max) {
      result.push_back(point_ids_x_sorted_[x_min_node.pos()]);
    }
    return result;
  }

  // general case: run up the tree in both paths
  y_query(R, x_min_node, result);
  y_query(R, x_max_node, result);
  while (x_min_node.parent().pos() != x_max_node.parent().pos()) {
    if (x_min_node.is_left_child()) {
      TreeNode sibling = x_min_node.parent().right_child();
      y_query(R, sibling, result);
    }
    if (x_max_node.is_right_child()) {
      TreeNode sibling = x_max_node.parent().left_child();
      y_query(R, sibling, result);
    }
    x_min_node = x_min_node.parent();
    x_max_node = x_max_node.parent();
  }

  return result;
}

TreeNode RangeTree::x_query(int x_coord, bool leftmost) const {
  TreeNode node(0, 0);
  while (node.layer() != max_layer()) {
    TreeNode left_child = node.left_child();
    TreeNode right_child = node.right_child();
    if (!right_child.node_exists(n())) {
      node = left_child;
      continue;
    }
    if (leftmost) {
      int median_pos = left_child.rightmost_leaf(n()).pos();
      if (x_coord <= points_[point_ids_x_sorted_[median_pos]].x) {
        node = left_child;
      } else {
        node = right_child;
      }
    } else {
      int median_pos = right_child.leftmost_leaf(n()).pos();
      if (x_coord >= points_[point_ids_x_sorted_[median_pos]].x) {
        node = right_child;
      } else {
        node = left_child;
      }
    }
  }
  return node;
}

void RangeTree::y_query(const Rectangle& R, TreeNode node,
                        std::vector<int>& result) const {
  const std::vector<int>& y_array =
      point_ids_y_sorted_[node.layer()][node.pos()];
  auto lower = std::lower_bound(
      y_array.begin(), y_array.end(), R.y_min,
      [&](int id, int y_min) { return points_[id].y < y_min; });
  auto upper = std::upper_bound(
      y_array.begin(), y_array.end(), R.y_max,
      [&](int y_max, int id) { return y_max < points_[id].y; });

  for (; lower != upper; ++lower) {
    result.push_back(*lower);
  }
}

//////////////////////////////////////////////////////////////////////
// some output stuff

std::ostream& operator<<(std::ostream& os, const Point& p) {
  return os << "(" << p.x << ", " << p.y << ")";
  return os;
}

void RangeTree::print() const {
  TreeNode node(0, 0);
  while (node.layer() <= max_layer()) {
    int max_pos = std::pow(2, node.layer());
    while (node.pos() < max_pos && node.node_exists(n())) {
      std::cout << "layer = " << node.layer() << " pos = " << node.pos();
      std::cout << " y-array = ";
      for (int id : point_ids_y_sorted_[node.layer()][node.pos()]) {
        std::cout << points_[id] << " ";
      }
      std::cout << std::endl;
      node = TreeNode(node.layer(), node.pos() + 1);
    }
    std::cout << std::endl;
    node = TreeNode(node.layer() + 1, 0);
  }
}

//////////////////////////////////////////////////////////////////////
// main

int main(int argc, char* argv[]) {

  std::ios_base::sync_with_stdio(false);
  std::vector<Point> points;

  int n, k;

  std::cin >> n;
  std::cin >> k;

  int id, x, y;
  for (int i = 0; i < n; ++i) {
    std::cin >> id >> x >> y;

    Point p = {x, y};
    points.push_back(p);
  }

  // building the range tree
  RangeTree T(points);

  int x_min, x_max, y_min, y_max;
  for (int i = 0; i < k; ++i) {

    std::cin >> x_min >> x_max >> y_min >> y_max;

    Rectangle R = {x_min, x_max, y_min, y_max};
    std::vector<int> result = T.range_query(R);
    std::sort(result.begin(), result.end());

    for (unsigned int i = 0; i < result.size(); ++i) {
      if (i != 0) std::cout << "\t";
      std::cout << result[i];
    }
    std::cout << "\n";
  }

  return 0;
}
